package com.utopiaindustries.hseobservationsapp.activities.ObservationForms;

import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;
import androidx.lifecycle.ViewModelProvider;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.utopiaindustries.hseobservationsapp.R;
import com.utopiaindustries.hseobservationsapp.activities.DashboardActivity;
import com.utopiaindustries.hseobservationsapp.activities.LoginActivity;
import com.utopiaindustries.hseobservationsapp.helper.Helper;
import com.utopiaindustries.hseobservationsapp.helper.Preference;
import com.utopiaindustries.hseobservationsapp.utils.HSERequestModel;
import com.utopiaindustries.hseobservationsapp.utils.ObservationStorageManager;
import com.utopiaindustries.hseobservationsapp.utils.ProgressDialogFragment;
import com.utopiaindustries.hseobservationsapp.viewmodels.LoginViewModel;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.Map;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;


public class ObservationThreeActivity extends AppCompatActivity {

    RadioGroup rg1;
    ImageView imgBack;
    Button btnSubmit, btnDraft;

    LoginViewModel loginViewModel;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        setContentView(R.layout.activity_observation_three);
        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });

        initializeLayouts();

        imgBack.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        rg1.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener()
        {
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                if (checkedId == R.id.rb_open) {
                    ObservationStorageManager.getInstance().setObservationStatus(true);
                } else if (checkedId == R.id.rb_closed) {
                    ObservationStorageManager.getInstance().setObservationStatus(false);
                }
            }
        });

        btnSubmit.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                alertReportSubmit(ObservationThreeActivity.this);
                /*Toast.makeText(ObservationThreeActivity.this,"Reported Submitted",Toast.LENGTH_SHORT).show();
                Intent intent = new Intent(ObservationThreeActivity.this, DashboardActivity.class);
                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_NEW_TASK);
                intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                startActivity(intent);
                overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);*/
            }
        });

        btnDraft.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });
    }

    private void initializeLayouts() {

        rg1 = findViewById(R.id.rg1);
        imgBack = findViewById(R.id.img_back);

        btnSubmit = findViewById(R.id.btn_submit);
        btnDraft = findViewById(R.id.btn_draft);

        loginViewModel = new ViewModelProvider(this).get(LoginViewModel.class);

        loginViewModel.getLoadingState().observe(this, isLoading -> {

            if (isLoading != null && isLoading) {
                showProgressDialog();
            } else {
                dismissProgressDialog();
            }
        });

        loginViewModel.getErrorMessage().observe(this, errorResponse -> {
            Toast.makeText(this, errorResponse, Toast.LENGTH_SHORT).show();
        });

        loginViewModel.getUserSaveLiveData().observe(this, hseSaveResponse -> {
            if (hseSaveResponse != null && hseSaveResponse.getStatus().equals("success")) {
                Toast.makeText(this, "Reported Submitted", Toast.LENGTH_SHORT).show();
                Intent intent = new Intent(ObservationThreeActivity.this, DashboardActivity.class);
                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_NEW_TASK);
                intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                startActivity(intent);
                overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);
            }
        });
    }

    public void alertReportSubmit(Context con) {
        ViewGroup viewGroup = findViewById(android.R.id.content);

        TextView dialogOkBtn, dialogCancelBtn;

        AlertDialog.Builder builder = new AlertDialog.Builder(con);
        View view1 = LayoutInflater.from(con).inflate(R.layout.custom_layout_for_report_submission, viewGroup, false);
        builder.setCancelable(false);
        builder.setView(view1);

        dialogOkBtn = view1.findViewById(R.id.dialogOkBtn);
        dialogCancelBtn = view1.findViewById(R.id.dialogCancelBtn);

        AlertDialog alertDialog = builder.create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

        dialogOkBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                alertDialog.dismiss();

               // byte[] reportBytes = convertReportToByteArray();
                HSERequestModel requestModel = new HSERequestModel();
                requestModel.setSite_id(ObservationStorageManager.getInstance().getSiteId());
                requestModel.setSupervisorId(ObservationStorageManager.getInstance().getSupervisorId());
                requestModel.setObservation_date(ObservationStorageManager.getInstance().getReportDate());
                requestModel.setShift(ObservationStorageManager.getInstance().getShiftName());
                requestModel.setRecordTypeId(ObservationStorageManager.getInstance().getRecordTypeId());
                requestModel.setHseReportObservationRecord(ObservationStorageManager.getInstance());

                // gson = new GsonBuilder().setPrettyPrinting().create();
                //String jsonRequest = gson.toJson(requestModel);
               // Log.e("RequestModel JSON", requestModel.toString());

                loginViewModel.saveHSEData(requestModel);


            }
        });

        dialogCancelBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                alertDialog.dismiss();
                //Toast.makeText(con, "Cancel", Toast.LENGTH_SHORT).show();

            }
        });

        alertDialog.show();
    }

    private byte[] convertReportToByteArray() {
        byte[] bytes = new byte[0];
        ByteArrayOutputStream byteArrayOutputStream = null;
        ObjectOutputStream objectOutputStream = null;
        try {
            ObservationStorageManager observationStorageManager = ObservationStorageManager.getInstance();
            // convert into byte array
            byteArrayOutputStream = new ByteArrayOutputStream();
            objectOutputStream = new ObjectOutputStream( byteArrayOutputStream );
            objectOutputStream.writeObject( observationStorageManager );
            objectOutputStream.flush();
            bytes =  byteArrayOutputStream.toByteArray();

        } catch (  Exception  e ) {
            e.printStackTrace();
        }
        finally {
            try {
                if (objectOutputStream != null) objectOutputStream.close();
                if (byteArrayOutputStream != null) byteArrayOutputStream.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }

        return bytes;
    }

    public void showProgressDialog() {
        ProgressDialogFragment progressDialog = new ProgressDialogFragment();
        progressDialog.setCancelable(false);
        progressDialog.show(getSupportFragmentManager(), "progressDialog");
    }

    public void dismissProgressDialog() {
        ProgressDialogFragment progressDialog = (ProgressDialogFragment)
                getSupportFragmentManager().findFragmentByTag("progressDialog");
        if (progressDialog != null) {
            progressDialog.dismiss();
        }
    }
}